#include "InertiaTypeInfo.h"

namespace RTT
{

// << >> operators must be in the RTT namespace
std::ostream& operator<<(std::ostream& os, const TypekitTest::Inertia& inertia)
{
    return os << "[x=" << inertia.ix << ", y=" << inertia.iy << ", z=" << inertia.iz << ", xy=" << inertia.ixy << ", xz=" << inertia.ixz << ", yz=" << inertia.iyz << "]";
}

std::istream& operator>>(std::istream& is, TypekitTest::Inertia& inertia)
{
    char c;
    return is >> c >> inertia.ix >> c >> inertia.iy >> c >> inertia.iz >> c >> inertia.ixy >> c >> inertia.ixz >> c >> inertia.iyz >> c;
}

} // namespace RTT

namespace TypekitTest
{

TypekitTest::Inertia createInertia(double ix, double iy, double iz, double ixy, double ixz, double iyz)
{
    return TypekitTest::Inertia(ix,iy,iz,ixy,ixz,iyz);
}

bool InertiaTypeInfo::decomposeTypeImpl(const TypekitTest::Inertia inertia, PropertyBag& targetbag) const
{
    targetbag.setType("Inertia");

    if(!targetbag.empty())
        return false;

    targetbag.add(new Property<double>("x","Ix",inertia.ix));
    targetbag.add(new Property<double>("y","Iy",inertia.iy));
    targetbag.add(new Property<double>("z","Iz",inertia.iz));
    targetbag.add(new Property<double>("xy","Ixy",inertia.ixy));
    targetbag.add(new Property<double>("xz","Ixz",inertia.ixz));
    targetbag.add(new Property<double>("yz","Iyz",inertia.iyz));

    return true;
}

bool InertiaTypeInfo::composeTypeImpl(const PropertyBag& bag, TypekitTest::Inertia inertia) const
{
    Property<double> prop;

    if(bag.getType() == "Inertia")
    {
        if(bag.size() != 6)
        {
            log(Error) << "Can't compose Inertia type: missing elements in PropertyBag." << endlog();
            return false;
        }

        prop = bag.getProperty("x");

        if(prop.ready())
        {
            inertia.ix = prop.get();
        }
        else
        {
            log(Error) << "Can't compose Inertia type: missing element 'x' in PropertyBag." << endlog();
            return false;
        }

        prop = bag.getProperty("y");

        if(prop.ready())
        {
            inertia.iy = prop.get();
        }
        else
        {
            log(Error) << "Can't compose Inertia type: missing element 'y' in PropertyBag." << endlog();
            return false;
        }

        prop = bag.getProperty("z");

        if(prop.ready())
        {
            inertia.iz = prop.get();
        }
        else
        {
            log(Error) << "Can't compose Inertia type: missing element 'z' in PropertyBag." << endlog();
            return false;
        }

        prop = bag.getProperty("xy");

        if(prop.ready())
        {
            inertia.ixy = prop.get();
        }
        else
        {
            log(Error) << "Can't compose Inertia type: missing element 'xy' in PropertyBag." << endlog();
            return false;
        }

        prop = bag.getProperty("xz");

        if(prop.ready())
        {
            inertia.ixz = prop.get();
        }
        else
        {
            log(Error) << "Can't compose Inertia type: missing element 'xz' in PropertyBag." << endlog();
            return false;
        }

        prop = bag.getProperty("yz");

        if(prop.ready())
        {
            inertia.iyz = prop.get();
        }
        else
        {
            log(Error) << "Can't compose Inertia type: missing element 'yz' in PropertyBag." << endlog();
            return false;
        }
    }
    else
    {
        return false;
    }

    return true;
}

}

